using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using SubSystems.TE;
using SubSystems.HT;

namespace Demo
{
	/// <summary>
 /// Summary description for demo.
	/// </summary>
 public class Demo : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

  ////////////////////////////////////////////////////////////////////////////////////////////////////
  ///    Demo Variable declarations
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  internal const int MAX_CTLS = 10;  // max controls to be open

  internal struct StrCtl {
   internal bool InUse;      // slot in use
   internal int x;           // x location
   internal int y;           // y location
   internal int width;       // width of the control
   internal int height;      // height of the control
   internal bool FittedView; // FittedView mode
   internal bool VertScrollBar; // show vertical toolbar
   internal bool HorzScrollBar; // show the horizontal scroll bar
   internal bool ShowMenu;    // show the menu bar
   internal bool BorderMargin;// create the border margin
   internal bool EditMode;    // true for edit mode

   internal Form form;        // form containing the TE control
   internal Tern toc;         // TE control
   internal Htn  htn;         // htn object for TE control

   internal char InputType;   // 'F' file, 'B' buffer
   internal string file;      // file-name to open
  };

  internal StrCtl[] ctl=new StrCtl[MAX_CTLS];
  internal int CurCtl;       // current control
  internal Tern CurToc;      // current toc control

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///    Constructor
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  public Demo()
		{
    int i;
    //
    // Required for Windows Form Designer support
    //
    InitializeComponent();

    //Application.EnableVisualStyles();
    //Application.SetCompatibleTextRenderingDefault(false);
    
    //  Build and attach a menu
    this.Menu=BuildMenu();

    // initialize StrCtl structure
    for (i=0;i<MAX_CTLS;i++) {
      ctl[i]=new StrCtl();
      ctl[i].x=10;
      ctl[i].y=50;
      ctl[i].width=650;
      ctl[i].height=850;
      ctl[i].FittedView=true;
      ctl[i].ShowMenu=true;
      ctl[i].VertScrollBar=true;
      ctl[i].HorzScrollBar=false;
      ctl[i].BorderMargin=true;
      ctl[i].EditMode=false;
      ctl[i].InputType='F';
      ctl[i].file="demo.htm";
    } 
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// Demo
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
   this.ClientSize = new System.Drawing.Size(300, 300);
			this.Name = "Demo";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
   this.Text = "HTML Add-on";

		}
		#endregion
 
  /// <summary>
  /// The main entry point for the application.
  /// </summary>
  [STAThread]
  static void Main() 
  {
   Application.Run(new Demo());
  }

  protected override void OnPaintBackground(System.Windows.Forms.PaintEventArgs ev)
  {
    base.OnPaintBackground(ev);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //    Helper functions                                                                    
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /*****************************************************************************************************************
  SelectControl:  
  Select an open control for processing.  This function returns no open controls are found
  *****************************************************************************************************************/
  internal int SelectControl()
  {
     int i,OpenCount;

     OpenCount=0;
     CurCtl=-1;
     for (i=0;i<MAX_CTLS;i++) {
        if (ctl[i].InUse && ctl[i].toc!=null) {
           CurCtl=i;
           OpenCount++;
        } 
     }
     if (OpenCount<=1) return CurCtl;
     
     // Show a dialog box to select from an open control
     CurCtl=-1;
     DmoSelect DlgSelect = new DmoSelect(this);
     DialogResult result=DlgSelect.ShowDialog();
     DlgSelect.Dispose();

     return CurCtl;
  } 

  /////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //    Testing/Debugging code                                                                    
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /*****************************************************************************************************************
  DemoOther:  test code here
  *****************************************************************************************************************/
  internal void DemoOther(Form form, Tern tern, Htn htn)
  {
     //htn.HtsSetFlags(true,hc.HFLAG_PRESERVE_SPACES);
     htn.HtsSetFlags2(true,hc.HFLAG2_WRITE_FIXED_TABLE_LAYOUT);
  }

  /***************************************************************************
      ToInt:
      Convert string to integer
  ****************************************************************************/
  internal new int ToInt(TextBox item)
  {
     return ToInt(item.Text);
  }

  internal new int ToInt(string txt)
  {
     try {
        return (int)Convert.ToInt32(txt);
     } 
     catch (System.Exception)
     {
        return 0;
     }
  } 

  internal new bool True(int val)
  {
     return (val!=0);
  }
    
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //    Menu Functions                                                                    
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////
  public const int IDM_OPEN        =1;
  public const int IDM_OTHER       =3;
  public const int IDM_SHOW_FRAMES = 8;
  public const int IDM_DEBUG       = 12;
  /*****************************************************************************************************************
  MenuClick:  Execute menu click
  *****************************************************************************************************************/
  internal void MenuClick(object sender, System.EventArgs ev)
  {
     int i;
     int CmdId= ((OurMenuItem)sender).CmdId;

     if (CmdId==IDM_OPEN) {
        for (i=0;i<MAX_CTLS;i++) if (!ctl[i].InUse) break;
        if (i==MAX_CTLS) {
           MessageBox.Show("Ran out of open slots!","Error",MessageBoxButtons.OK);
           return;
        } 
        CurCtl=i;

        // select the open parameters
        DmoOpen DlgOpen = new DmoOpen(this);
        DialogResult result=DlgOpen.ShowDialog();
        DlgOpen.Dispose();
        if (result==DialogResult.Cancel) return;

        if (!ctl[CurCtl].InUse) return;    

        // Open a form containing TE
        DmoTe FormTe = new DmoTe(this);
        FormTe.Show();
     }
     else if (CmdId==IDM_SHOW_FRAMES) {
        CurCtl=SelectControl();
        if (CurCtl>=0) ctl[CurCtl].toc.TerSetFlags3(true,tc.TFLAG3_SHOW_FRAMES);
     }
     else if (CmdId==IDM_DEBUG) {
        tc.DebugMode=!tc.DebugMode;
     }
     else if (CmdId==IDM_OTHER) {
        CurCtl=SelectControl();
        if (CurCtl>=0) DemoOther(ctl[CurCtl].form,ctl[CurCtl].toc,ctl[CurCtl].htn);
        else           DemoOther(null,null,null);
     }
     
     if (CurCtl>=0 && ctl[CurCtl].InUse) ctl[CurCtl].toc.Focus(); 
  } 

  /*****************************************************************************************************************
  SubmenuPopup:  Execute menu popup event
  *****************************************************************************************************************/
  internal void SubmenuPopup(object sender, System.EventArgs ev)
  {
     OurMenuItem SubMenu = (OurMenuItem) sender;
     Tern toc=null;

     if (CurCtl<0 || !ctl[CurCtl].InUse) CurCtl=SelectControl();
     if (CurCtl>=0) toc=ctl[CurCtl].toc;
     if (toc==null) return;

     foreach (MenuItem mItem in SubMenu.MenuItems) {
       if (mItem.Text=="-") continue;  // separator line

       OurMenuItem item = (OurMenuItem) mItem;
       
       int CmdId=item.CmdId;
       if (CmdId==IDM_DEBUG) {
         item.Enabled=toc.TerMenuEnable2(tc.ID_FITTED_VIEW);
         item.Checked=tc.DebugMode;
       }
     }   
  } 


  /************************************************************************************************************
     BuildMenu:
     Build menu for the demo program
  ************************************************************************************************************/
  public new MainMenu BuildMenu()
  {
     MainMenu menu=new MainMenu();

     // Open menu 
     AddMenuItem(menu.MenuItems,"Open"  ,IDM_OPEN,false ,(Shortcut)0);

     // add Other submenu
     OurMenuItem Other = AddSubMenu(menu.MenuItems,"Other");

     AddMenuItem(Other.MenuItems,"Debug Mode"  ,IDM_DEBUG,false ,(Shortcut)0);
     AddMenuItem(Other.MenuItems,"Show Frames" ,IDM_SHOW_FRAMES,false ,(Shortcut)0);
     AddMenuItem(Other.MenuItems,"Other"       ,IDM_OTHER,false ,(Shortcut)0);

     return menu;
  } 

  /*****************************************************************************************************************
  AddSubMenu:  Add a sub menu
  *****************************************************************************************************************/
  internal OurMenuItem AddSubMenu(Menu.MenuItemCollection MenuItems,string text)
  {
      OurMenuItem item = new OurMenuItem();
      
      item.Text=text;
      item.Popup+=new EventHandler(SubmenuPopup);
      item.CmdId=0;
      MenuItems.Add(item);

      return item;
  } 

  /*****************************************************************************************************************
  AddMenuItem:  Add a menu item
  *****************************************************************************************************************/
  internal OurMenuItem AddMenuItem(Menu.MenuItemCollection MenuItems,string text, int CmdId,bool ShowShortcut,Shortcut shortcut)
  {
      OurMenuItem item = new OurMenuItem();
      
      item.Text=text;
      item.Click+=new EventHandler(MenuClick);
      item.CmdId=CmdId;
      item.ShowShortcut=ShowShortcut;
      if ((int)shortcut!=0) item.Shortcut=shortcut;
      

      MenuItems.Add(item);

      return item;
  } 
  
  /*****************************************************************************************************************
  AddSeparator:  Add a menu item separator
  *****************************************************************************************************************/
  internal void AddSeparator(MenuItem parent)
  {
      MenuItem item = new MenuItem("-");
      
      if (parent!=null) parent.MenuItems.Add(item);
  }


 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 /*  Class OurMenuItem:  Adds new members */
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////

 class OurMenuItem: MenuItem
 {
    internal int CmdId;      // command id associated with this menu item
 } 

}
